@@ -35,7 +35,7 @@ gboolean                 photos_thumbnail_factory_generate_thumbnail     (Photos
                                                                           GQuark orientation,
                                                                           gint64 original_height,
                                                                           gint64 original_width,
-                                                                          const gchar *pipeline_uri,
+                                                                          const gchar *const *pipeline_uris,
                                                                           const gchar *thumbnail_path,
                                                                           GCancellable *cancellable,
                                                                           GError **error);
